// @flow
type ConstructorOptions = {
  canvasSelector?: string,
  rows: number,
  columns: number };

export default class SpikeWarsArtist {
  canvas: HTMLCanvasElement;

  // This is a reference to the drawing API: how we tell the computer to draw
  // on the canvas. It is a 2D context, meaning we draw in two-dimensions, like
  // on paper.
  canvas2d: CanvasRenderingContext2D;

  rows: number;       // How many rows are on our game board?
  cols: number;       // How many columns are on our game board?
  tileWidth: number;  // How wide is one of our game board spots?
  tileHeight: number; // How tall is one of our game bouard spots?

  constructor({canvasSelector = 'canvas', rows, columns}: ConstructorOptions) {
    this.canvas = ((document.querySelector(canvasSelector): any): HTMLCanvasElement);

    var c2d: ?CanvasRenderingContext2D = this.canvas.getContext('2d');
    if (c2d) this.canvas2d = c2d;
    else throw "Cannot get 2D rendering context!"; // TODO: better handling

    this.rows = rows;
    this.cols = columns;

    this.tileHeight = Math.ceil(this.canvas.height / this.rows);
    this.tileWidth = Math.ceil(this.canvas.width / this.cols);

    console.log("SpikeWarsArtist: \n\trows: ", this.rows, "\tcols: ", this.cols,
                "\n\ttileWidth: ", this.tileWidth, "\ttileHeight: ", this.tileHeight);
  }

  coord2idx(row: number, col: number): number { return (this.cols * row) + col; }
  idx2row(idx: number): number { return Math.floor(idx/this.cols); }
  idx2col(idx: number): number { return idx % this.cols; }

  drawSolidColor(row: number, col: number, color: string): void {
    this.drawSolidColorIdx(this.coord2idx(row, col), color); }

  drawSolidColorIdx(idx: number, color: string): void {
    this.canvas2d.fillStyle = color;

    this.canvas2d.fillRect(
      (idx % this.cols) * this.tileWidth,
      Math.floor(idx / this.cols) * this.tileHeight,
      this.tileWidth, this.tileHeight); }
 
}
